

'''
	https://stackoverflow.com/questions/15562446/how-to-stop-flask-application-without-using-ctrl-c
'''

#
#	https://stackoverflow.com/questions/2470971/fast-way-to-test-if-a-port-is-in-use-using-python
#
def is_port_in_use (port: int) -> bool:
	import socket
	with socket.socket (socket.AF_INET, socket.SOCK_STREAM) as s:
		return s.connect_ex (('localhost', port)) == 0

import flask
from flask import Flask
from multiprocessing import Process
import pathlib
from os.path import dirname, join, normpath
THIS_FOLDER = pathlib.Path (__file__).parent.resolve ()

import shares.basin.treasury as treasury

def start (
	paths = [],
	start_at_port = 2345,
	directory = ""
):
	app = Flask (__name__)

	treasury_string = treasury.start (
		links = paths
	)

	@app.route ("/")
	def treasury_route ():
		return treasury_string
	
	@app.route ("/<path:path>")
	def page (path):
		print (path)
		
		for found_path in paths:
			if (found_path ['path'] == path):
				return "".join (
					open (found_path ['find'], "r").readlines ()
				)
	
		return 'not found'
	

	
	

	'''
	server = Process (
		target = app.run,
		args = (),
		kwargs = {
			"port": 2345
		}
	)
	
	
	return server;
	'''

	'''
	server.terminate ()
	server.join ()
	'''
	'''
	def run_v1 (limit, loop):
		print (f"run attempt { loop } of { limit }")
	
		try:
			port = start_at_port - 1 + loop
			unavailable = is_port_in_use (port)
			#print ("unavailable:", unavailable)
			
			if (unavailable):
				raise Exception ("unavailable")
		
			app.run (
				port = port
			)
		
			print ('shares app started')
			return			
		except Exception as E:
			pass;
			
		loop += 1;	
		
		run (
			limit,
			loop = loop
		)
	'''
	
	def run (limit, loop):
		try:
			port = start_at_port - 1 + loop
			print (f"run attempt { loop } of { limit }:", port)
			
			unavailable = is_port_in_use (port)
			#print ("unavailable:", unavailable)
			
			if (unavailable):
				raise Exception ("unavailable")
		
			server = Process (
				target = app.run,
				args = (),
				kwargs = {
					"port": port
				}
			)
		
			print ('shares app started')
			return {
				"server": server,
				"port": port
			}
			
		except Exception as E:
			pass;
			
		loop += 1;	
		
		return run (
			limit,
			loop = loop
		)
			
		
	return run (
		limit = 100,
		loop = 1
	)





from mako.template import Template

def start (
	links
):
	'''
	links = [
		{
			"path": "company/company.s.HTML",
			"name": "company/company"
		}
	]
	'''
	
	if (len (links) >= 1):
		open = links [0]['path']
	else:
		open = ""
	
	def make_links ():
		links_string = ""
		for link in links:
			path = link ['path']
			name = link ['name']
		
			links_string += (
				f'<a href="{ path }">{ name }</a>'
			)
			
		return links_string
		
	def make_select ():
		options = ""
		for link in links:
			path = link ['path']
			name = link ['name']
		
			options += (
				f'<option value="{ path }">{ name }</a>'
			)
			
		return """
			<select 
				id='link_selector'
				style='
					font-size: 1.2em;
				'
			>
		""" + options + "</select>"
		
	links_selector = make_select ()	
	links_string = make_links ()


	temp = Template ('''
<!doctype html>
<html lang="en">
<head>
	<meta charset="UTF-8" />
	<style>
	code {
		display: block;
		background: #EEE;
		padding: 10px;
		border-radius: 4px;
	}
	
	a {
		display: block;
	}
	

	</style>
</head>
<body 
	style="
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
	
		display: flex;
		flex-direction: column;
	
		margin: 0;
		padding: 0.2in; 
		
		font-size: 1.3em; 
		opacity: 0; 
		transition: opacity .5s;
	"
>
	<div
		style="
			position: absolute;
			left: 0;
			top: 0;
		
			display: flex;
			justify-content: space-between;
			align-items: center;
			
			width: 100%;
			height: .75in;
			
			box-sizing: border-box;
			padding: 0 .25in;
			
			overflow: visible;
		"
	>
		<b>{ shares }</b>			
		<div>${ links_selector }</div>
	</div>
	<div 
		style="
			position: absolute;
			left: 0;
			top: .75in;
		
			width: 100%;
			height: calc(100% - .75in);
		
			box-sizing: border-box;
			padding: .2in;
			
		"
	>
		<iframe 
			id="statement"
			style="
				width: 100%;
				height: 100%;
				border: 0;
				
				opacity: 0;
				transition: opacity .1s;
				
				box-sizing: border-box;
				box-shadow: 0 0 9px 1px #bbb;
				border-radius: 6px;
			"
			src="${ open }" 
		></iframe>
	</div>

	<script>
		console.log ("lyrics");
		
		/*
					
		*/
		document.addEventListener ("DOMContentLoaded", function(event) { 
			document.body.style.opacity = 1;
			
			setTimeout (() => {		
				document.body.style.opacity = 1;
			}, 500)
		});	

		const $__statement = document.getElementById ("statement")

		link_selector = document.getElementById ("link_selector")
		console.log (link_selector)
		
		link_selector.addEventListener ("change", function (event) {
			console.log (event)
			$__statement.style.opacity = 0;
			
			setTimeout (() => {	
				$__statement.src = event.target.value
			}, 120);
			
			setTimeout (() => {	
				$__statement.style.opacity = 1;
			}, 140)
		})

		document.querySelectorAll ('a').forEach ($__link => {
			console.log ("for each", $__link)
			
			$__link.addEventListener ("click", function ($__event) {
				$__event.preventDefault ()
				$__statement.style.opacity = 0;
	
				// console.log (this.href)

				setTimeout (() => {	
					$__statement.src = this.href
				}, 120);
				
				setTimeout (() => {	
					$__statement.style.opacity = 1;
				}, 140)
			})
		})
		
		
		document.addEventListener ("DOMContentLoaded", function(event) { 
			$__statement.style.opacity = 1;
			setTimeout (() => {		
				$__statement.style.opacity = 1;
			}, 500)
		});	
	</script>
</body>
</html>
''')

	return temp.render (
		links_string = links_string,
		links_selector = links_selector,
		open = open
	)




'''
	import shares
	shares = shares.start ()
	
	shares.server.stop ()
'''

import glob
import inspect
import json
import os
import pathlib
import rich

import shares.basin as basin
from shares._clique import clique

import shares.modules.law_dictionary as law_dictionary

def retrieve_directory ():
	return os.getcwd ()
	
	return os.path.dirname (
		os.path.abspath ((inspect.stack ()[1]) [1])
	)

	
def start (
	params = {},
	verbose = 1
):	
	if (verbose >= 1):
		print ()
		print ('"shares" is starting.')
		print ('module path is:', pathlib.Path (__file__).parent.resolve ())
		print ()


	law_dictionary.check (	
		allow_extra_fields = True,
		laws = {
			"directory": {
				"required": False,
				"contingency": retrieve_directory,
				"type": str
			},
			"relative path": {
				"required": False,
				"contingency": retrieve_directory,
				"type": str
			},
			"extension": {
				"required": False,
				"contingency": ".s.HTML",
				"type": str
			},
			"port": {
				"required": False,
				"contingency": 2345,
				"type": int
			},
			"static port": {
				"required": False,
				"contingency": False,
				"type": bool 
			},
			"verbose": {
				"required": False,
				"contingency": 1,
				"type": int
			}
		},
		dictionary = params
	)
	
	extension = params ["extension"]
	directory = params ["directory"]
	relative_path = params ["relative path"]

	rich.print_json (data = params)

	glob_param = directory + "/**/*" + extension;
	if (verbose >= 2):
		print ("glob:", glob_param)

	finds = glob.glob (glob_param, recursive = True)
	if (verbose >= 2):
		print ("finds:", json.dumps (finds, indent = 4))
	
	paths = []
	for find in finds:
		path = os.path.relpath (find, relative_path)
		name = path.split (extension) [0]
	
		paths.append ({
			"path": path,
			"name": name,
			"find": find
		})
	
	if (verbose >= 1):
		print ("paths:", json.dumps (paths, indent = 4))
	
	the_server = basin.start (
		paths = paths,
		directory = directory
	)
	
	server = the_server ["server"]
	actual_port = the_server ["port"]
	
	server.start ()
	
	def stop ():
		if (verbose >= 1):
			print ('"shares" is stopping.')
	
		
		#server.stop ()
		termination = server.terminate ()
		#server.join ()
	
		if (verbose >= 1):
			print ('"shares" has stopped.')
			print ()
	
	import atexit
	atexit.register (stop)
	
	print ('The shares server has started')

	class Proceeds ():
		def __init__ (this, server, stop, port):
			this.stop = stop;
			this.port = port;
			

	return Proceeds (
		server,
		stop,
		actual_port
	)